@@ -1,28 +1,39 @@
 package Catalyst::View::JavaScript::Minifier::XS;
 BEGIN {
-  $Catalyst::View::JavaScript::Minifier::XS::VERSION = '2.000000';
+  $Catalyst::View::JavaScript::Minifier::XS::VERSION = '2.100000';
 }
 
 # ABSTRACT: Minify your served JavaScript files
 
 use autodie;
 use Moose;
+
 extends 'Catalyst::View';
 
 use JavaScript::Minifier::XS qw/minify/;
-use Path::Class::File;
+use Moose::Util::TypeConstraints;
+use MooseX::Aliases;
+use Path::Class::Dir;
 use URI;
 
+my $dir_type = __PACKAGE__.'::Dir';
+subtype $dir_type => as class_type('Path::Class::Dir');
+coerce $dir_type,
+    from 'Str',      via { Path::Class::Dir->new($_)  },
+    from 'ArrayRef', via { Path::Class::Dir->new(@$_) };
+
 has stash_variable => (
    is => 'ro',
    isa => 'Str',
    default => 'js',
 );
 
-has path => (
-   is => 'ro',
-   isa => 'Str',
+has js_dir => (
+   is      => 'ro',
+   isa     => $dir_type,
+   coerce  => 1,
    default => 'js',
+   alias   => 'path',
 );
 
 has subinclude => (
@@ -31,6 +42,13 @@ has subinclude => (
    default => undef,
 );
 
+# for backcompat. don't use this.
+has 'INCLUDE_PATH' => (
+    is     => 'ro',
+    isa    => $dir_type,
+    coerce => 1,
+   );
+
 sub process {
    my ($self,$c) = @_;
 
@@ -41,15 +59,20 @@ sub process {
 
    push @files, $self->_subinclude($c, $original_stash, @files);
 
-   my $home = $self->config->{INCLUDE_PATH} || $c->path_to('root');
+   # the 'root' conf var might not be absolute
+   my $abs_root = Path::Class::Dir->new( $c->config->{'root'} )->absolute( $c->path_to );
+   my $js_dir   = $self->js_dir->absolute( $abs_root );
+
+   # backcompat only
+   $js_dir = $self->INCLUDE_PATH->subdir($js_dir) if $self->INCLUDE_PATH;
+
    @files = map {
-      $_ =~ s/\.js$//;
-      Path::Class::File->new( $home, $self->path, "$_.js" );
+      $_ =~ s/\.js$//;  $js_dir->file( "$_.js" )
    } grep { defined $_ && $_ ne '' } @files;
 
-   my @output = $self->_combine_files($c, \@files);
+   my $output = $self->_combine_files($c, \@files);
 
-   $c->res->body( $self->_minify($c, \@output) );
+   $c->res->body( $self->_minify($c, $output) );
 }
 
 sub _subinclude {
@@ -82,7 +105,7 @@ sub _minify {
 
    if ( @{$output} ) {
       return $c->debug
-         ? join q{ }, @{$output}
+         ? join "\n", @{$output}
          : minify(join q{ }, @{$output} )
    } else {
       return q{ };
@@ -96,12 +119,10 @@ sub _combine_files {
    for my $file (@{$files}) {
       $c->log->debug("loading js file ... $file") if $c->debug;
       open my $in, '<', $file;
-      for (<$in>) {
-         push @output, $_;
-      }
-      close $in;
+      local $/;
+      push @output, scalar <$in>;
    }
-   return @output;
+   return \@output;
 }
 
 sub _expand_stash {
@@ -128,7 +149,7 @@ Catalyst::View::JavaScript::Minifier::XS - Minify your served JavaScript files
 
 =head1 VERSION
 
-version 2.000000
+version 2.100000
 
 =head1 SYNOPSIS
 
@@ -162,9 +183,10 @@ does not minify the javascript if the server is started in development mode.
 
 sets a different stash variable from the default C<< $c->stash->{js} >>
 
-=item path
+=item js_dir
 
-sets a different path for your javascript files
+Directory containing your javascript files.  If a relative path is
+given, it is taken as relative to your app's root directory.
 
 default : js
 
